package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;

import java.util.List;
import java.util.Map;

/**
 * 权限树
 * @ClassName: MenuApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/9/17 10:44 AM 
 */
public interface MenuApiService {
    /**
     * 根据list id查询
     * @Title: listByMenuIdList 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listByMenuIdList(List<Integer> menuIdList);
    /**
     * 查询用户拥有的所有菜单路由权限
     * @Title: listUserMenu 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listUserMenu(Integer userId);

    /**
     * 获取登陆用户的菜单(页面)
     * @param userId
     * @param currentMenu
     * @return
     */
    ServiceResponse<List<MenuDTO>> getUserMenuOfGic(Integer userId, Integer currentMenu);

    /**
     * 根据用户ID查询没拥有的gic操作项
     * @Title: getNoAuthOperationItemOfGic 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfGic(Integer userId);

    /**
     * 获取超级管理员菜单
     * @param userId
     * @return
     */
    ServiceResponse<List<MenuDTO>> getSuperAdminMenu(Integer userId);

    /**
     * 查询gic菜单页面树数据
     * @Title: listMenuTree 
     * @Description:
      * @author guojuxing  
     * @param params
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTreeForRole 
     * @Description:
      * @author guojuxing
     * @param project 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTreeForRole(String project);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTreeForRoleOfGic 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTreeForRoleOfGic();

    /**
     * 查询页面下面的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId);

    /**
     * 新增页面
     * @Title: saveGICPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> saveGICPage(MenuDTO menuDTO);


    /**
     * 查询已经购买的应用
     * @Title: listAppPageOfHasBuy 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<List<MenuDTO>> listAppPageOfHasBuy(Integer enterpriseId);

    /**
     * 编辑gic页面
     * @Title: updateGICPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateGICPage(MenuDTO menuDTO);
    /**
     * 删除页面、子页面、权限项、操作项
     * @Title: deleteMenu 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> deleteMenu(Integer menuId);
    /**
     * 新增GIC子页面
     * @Title: saveGICChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveGICChildPage(MenuDTO menuDTO);

    /**
     * 编辑gic子页面
     * @Title: updateGICChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO);
    /**
     * 编辑应用页面
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAppPage(MenuDTO menuDTO);
    /**
     * 新增应用子页面
     * @Title: saveAppChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveAppChildPage(MenuDTO menuDTO);
    /**
     * 编辑应用子页面
     * @Title: updateAppChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO);

    /**
     * 拖拽排序
     * @Title: setSort 
     * @Description:
      * @author guojuxing
     * @param id
     * @param sort 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> setSort(Integer id, Integer sort);

    /**
     * 新增操作项
     * @Title: saveOperationItem 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO);

    /**
     * 编辑操作项
     * @Title: updateOperationItem 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO);

    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 获取详情
     * @Title: getDetailById 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.MenuDTO> 
      */
    ServiceResponse<MenuDTO> getDetailById(Integer menuId);

    /**
     * 新增权限项
     * @Title: saveAuthItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params);

    /**
     * 编辑权限项
     * @Title: updateAuthItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params);

    /**
     * 查询页面下的权限项
     * @Title: listAuthItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuthItemListDTO>> 
      */
    ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId);
    /**
     * 获取gic基础服务的版本列表数据
     * @Title: getGicDefaultVersion 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.String>> 
      */
    ServiceResponse<Map<String, String>> getGicDefaultVersion();
}
