package com.gic.auth.dao.mapper;

import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabSysMenuMapper {
    /**
     * 根据主键删除
     *
     * @param menuId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer menuId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysMenu record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysMenu record);

    /**
     * 根据主键查询
     *
     * @param menuId 主键
     * @return 实体对象
     */
    TabSysMenu selectByPrimaryKey(Integer menuId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysMenu record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysMenu record);

    /**
     * 查询菜单树（页面、子页面、权限项）
     * @Title: listMenu 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenu(MenuListQO params);

    /**
     * 获取最大排序值
     * @Title: getMaxSortByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId  父级ID
     * @param project
     * @return  int 
      */
    int getMaxSortByParentId(@Param("parentId") Integer parentId, @Param("project") String project);
    /**
     * 查询页面下的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuItemByParentId(@Param("parentId") Integer parentId);

    /**
     * 查询页面下的菜单数据
     * @Title: listMenuByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuByParentId(@Param("parentId") Integer parentId, @Param("project") String project);
    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 查询
     * @Title: selectByIds 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByIds(@Param("menuIdList") List<Integer> menuIdList);

    /**
     * 查询页面下的权限项数据
     * @Title: listAuthItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  java.util.List<com.gic.auth.dto.AuthItemListDTO> 
      */
    List<AuthItemListDTO> listAuthItemByMenuId(@Param("menuId") Integer menuId);
    /**
     * 统计页面下的子项目
     * @Title: countChildByParentId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  int 
      */
    int countChildByParentId(@Param("menuId") Integer menuId);
    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTree 
     * @Description:
      * @author guojuxing
     * @param project 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuTree(@Param("project") String project);

    List<TabSysMenu> selectByProjectList(@Param("projectList")List<String> projectList);

    /**
     * 相同code个数
     * @Title: countByMenuCode 
     * @Description:
      * @author guojuxing
     * @param menuCode 
     * @return  int 
      */
    int countByMenuCode(@Param("menuCode") String menuCode,@Param("menuId") Integer menuId);

    /**
     * 批量更新子集的父级code数据，根据父级ID
     * @Title: updateParentCodeByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  void 
      */
    void updateParentCodeByParentId(@Param("parentId")Integer parentId, @Param("parentCode") String parentCode);

    /**
     * 仅超管可用的操作项
     * @Title: listAdminOnlyOperationItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysUser> listAdminOnlyOperationItem();
}