package com.gic.auth.service.impl;

import com.gic.auth.dao.mapper.TabSysUserResourceMapper;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.entity.TabSysUserResource;
import com.gic.auth.service.UserResourceService;
import com.gic.commons.util.EntityUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/29 10:03
 */
@Service
public class UserResourceServiceImpl implements UserResourceService {

    @Autowired
    private TabSysUserResourceMapper tabSysUserResourceMapper;

    @Override
    public boolean existUserResource(Integer enterpriseId, Integer resourceId) {
        TabSysUserResource userResource = tabSysUserResourceMapper.existUserResource(enterpriseId, resourceId);
        return userResource != null;
    }

    @Override
    public Map<Integer, Integer> countByResourceIds(Integer enterpriseId, List<Integer> resourceIds) {
        if (CollectionUtils.isNotEmpty(resourceIds)) {
            //
            Map<Integer, Integer> map = new HashMap<>();
            Map<Integer, Map<Integer, Long>> mapMap = tabSysUserResourceMapper.countByResourceIds(enterpriseId, resourceIds);
            for (Map.Entry<Integer, Map<Integer, Long>> entry : mapMap.entrySet()) {
                Integer resourceId = entry.getKey();
                map.put(resourceId, entry.getValue().get("userResourceCount").intValue());
            }
            return map;
        } else {
            return Collections.emptyMap();
        }
    }

    @Override
    public void save(UserResourceDTO dto) {
        tabSysUserResourceMapper.insert(EntityUtil.changeEntityNew(TabSysUserResource.class, dto));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteByUserId(Integer userId) {
        tabSysUserResourceMapper.deleteByUserId(userId);
    }

    @Override
    public List<UserResourceDTO> listUserResourceByUserId(Integer userId) {
        List<TabSysUserResource> list = tabSysUserResourceMapper.listUserResourceByUserId(userId);
        if (CollectionUtils.isNotEmpty(list)) {
            return EntityUtil.changeEntityListNew(UserResourceDTO.class, list);
        }
        return new ArrayList<>();
    }
}