package com.gic.auth.service.outer.impl;

import java.util.*;

import com.gic.open.api.service.ServeApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.*;
import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.entity.TabSysMenuItem;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.dto.ServePropDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.ApplicationApiService;

@Service("menuApiService")
public class MenuApiServiceImpl implements MenuApiService {
    private final static Logger LOGGER = LogManager.getLogger(MenuApiServiceImpl.class);
    @Autowired
    private MenuService menuService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private MenuItemService menuItemService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private UserService userService;
    @Autowired
    private MenuRoleService menuRoleService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private ServeApiService serveApiService;

    @Override
    public ServiceResponse<List<MenuDTO>> listByMenuIdList(List<Integer> menuIdList) {
        return ServiceResponse
                .success(EntityUtil.changeEntityListNew(MenuDTO.class, menuService.selectByIds(menuIdList)));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listUserMenu(Integer userId) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }

        List<MenuDTO> result;
        if (tabSysUser.getSuperAdmin().intValue() == 1) {
            List<TabSysMenu> temp = new ArrayList<>();
            //超级管理员
            MenuListQO params = new MenuListQO();
            params.setMenuType(MenuTypeEnum.PAGE.getCode());
            temp.addAll(this.menuService.listMenu(params));

            result = EntityUtil.changeEntityListNew(MenuDTO.class, temp);
        } else {
            List<TabSysMenu> menuList = getUserMenuList(userId);
            result = EntityUtil.changeEntityListNew(MenuDTO.class, menuList);
            ServiceResponse<List<MenuDTO>> tempResult = getNoAuthOperationItem(userId, MenuProjectConstants.ALL_CODE);
            result.addAll(tempResult.getResult());
        }

        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfGic(Integer userId, Integer currentMenu) {
        return getUserLoginMenu(userId, currentMenu, true);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfGic(Integer userId) {
        return getNoAuthOperationItem(userId, MenuProjectConstants.DEFAULT_PROJECT_CODE);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getSuperAdminMenu(Integer userId) {
        return this.getUserMenuOfGic(userId, null);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params) {
        if (params.getIsGIC() == null) {
            params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        }
        //应用菜单类型
        if (params.getIsGIC() == MenuProjectConstants.APP_PROJECT_CODE && params.isNeedUpdateAppMenu()) {
            saveAppMenu();
        }
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTreeForRole(String project) {
        List<TabSysMenu> list = menuService.listMenuTree(project);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTreeForRoleOfGic() {
        return listMenuTreeForRole(MenuProjectConstants.DEFAULT_PROJECT);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId) {
        TabSysMenu record = menuService.getMenuById(parentId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，页面不存在");
        }
        List<TabSysMenu> menuList = menuService.listMenuItemByParentId(parentId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<Void> saveGICPage(MenuDTO menuDTO) {
        menuDTO.setProject(MenuProjectConstants.DEFAULT_PROJECT);
        return savePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listAppPageOfHasBuy(Integer enterpriseId) {
        ServiceResponse<List<ApplicationDTO>> appResult = appTokenApiService
                .listApplicationByEnterpriseId(Long.valueOf(enterpriseId), null);
        if (appResult.isSuccess()) {
            List<ApplicationDTO> appList = appResult.getResult();
            if (CollectionUtils.isNotEmpty(appList)) {
                saveHasBuyAppMenu(appList);
                List<String> projectList = new ArrayList<>(appList.size());
                for (ApplicationDTO app : appList) {
                    projectList.add(app.getApplicationId().toString());
                }
                List<MenuDTO> menuList = EntityUtil.changeEntityListNew(MenuDTO.class,
                        menuService.selectByProjectList(projectList));
                return ServiceResponse.success(menuList);
            } else {
                return ServiceResponse.success(new ArrayList<>());
            }
        }
        return ServiceResponse.failure(appResult.getCode(), appResult.getMessage());
    }

    @Override
    public ServiceResponse<Void> updateGICPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> deleteMenu(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()) {
            //如果是页面，判断下面是否有子页面
            if (menuService.isHasChild(menuId)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面下有项目，不允许删除词页面");
            }
        } else if (record.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()) {
            //权限项，需要同时删除关联表
            menuItemService.deleteByMenuId(menuId);
        }

        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuId(menuId);
        menuDTO.setStatus(0);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveGICChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.UpdateGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppPage(MenuDTO menuDTO) {
        //只修改页面code
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getLevel().intValue() != MenuLevelConstants.FIRST_LEVEL) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
        }
        if (record.getMenuType().intValue() != MenuTypeEnum.PAGE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "主键参数有误，非" + MenuTypeEnum.PAGE.getMessage());
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, MenuDTO.UpdateAppPageValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), menuDTO.getMenuId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        menuService.updatePage(menuDTO);

        //同时更新子集的父级code数据
        menuService.updateParentCodeByParentId(menuDTO.getMenuId(), menuDTO.getMenuCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveAppChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.EditAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> setSort(Integer id, Integer sort) {
        TabSysMenu record = menuService.getMenuById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (sort == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值不能为空");
        }
        menuService.setSort(record, sort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.OPERATION.getCode());
        ServiceResponse<Integer> saveOperationItemResult = saveChildPage(menuDTO, MenuDTO.SaveOperationItemValid.class);
        if (saveOperationItemResult.isSuccess()) {
            if (menuDTO.getAdminOnlySign() != null && menuDTO.getAdminOnlySign().intValue() == MenuOperationItemConstants.ADMIN_ONLY) {
                //如果是仅超管可用，则不同步生成权限项
            } else if (menuDTO.isNeedCreateAuthItem()) {
                //同步生成权限项
                AuthItemSaveQO authItemSaveQO = new AuthItemSaveQO();
                authItemSaveQO.setMenuName(menuDTO.getMenuName());
                authItemSaveQO.setOperationItemIds(saveOperationItemResult.getResult().toString());
                authItemSaveQO.setParentId(menuDTO.getParentId());
                ServiceResponse saveAuthItemResult = saveAuthItem(authItemSaveQO);
                LOGGER.info("同步生成权限项结果:{}", JSON.toJSONString(saveAuthItemResult));
            }
        }
        return saveOperationItemResult;
    }

    @Override
    public ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.OPERATION, MenuDTO.UpdateOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params) {
        com.github.pagehelper.Page page = menuService.pageMenuOperationItem(params);
        Page<MenuDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, MenuDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<MenuDTO> getDetailById(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(MenuDTO.class, record));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params) {
        TabSysMenu record = menuService.getMenuById(params.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(params, AuthItemSaveQO.SaveAuthItemValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //新增权限项
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuName(params.getMenuName());
        menuDTO.setParentId(params.getParentId());
        menuDTO.setMenuType(MenuTypeEnum.ROLE.getCode());
        setNormalFieldValue(menuDTO, record);
        int menuId = menuService.savePage(menuDTO);

        String operationItemIds = params.getOperationItemIds();
        String[] operationItemArr = operationItemIds.split(SignConstants.COMMA);
        //校验操作项是否存在
        List<Integer> menuIdList = new ArrayList<>(operationItemArr.length);
        for (String str : operationItemArr) {
            menuIdList.add(Integer.parseInt(str));
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        if (menuList == null || menuList.size() < operationItemArr.length) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作项数据有误，其中有不存在的操作项");
        }
        //新增权限项、操作项关联表数据
        List<MenuItemDTO> itemDTOList = new ArrayList<>();
        Date now = new Date();
        for (String operationItem : operationItemArr) {
            MenuItemDTO menuItemDTO = new MenuItemDTO();
            menuItemDTO.setItemId(Integer.parseInt(operationItem));
            menuItemDTO.setMenuId(menuId);
            menuItemDTO.setStatus(1);
            menuItemDTO.setCreateTime(now);
            menuItemDTO.setUpdateTime(now);
            itemDTOList.add(menuItemDTO);
        }
        //delete
        menuItemService.deleteByMenuId(menuId);
        //save
        menuItemService.saveFetch(itemDTOList);
        return ServiceResponse.success();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params) {
        TabSysMenu record = menuService.getMenuById(params.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getMenuType().intValue() != MenuTypeEnum.ROLE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非权限项");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(params, AuthItemSaveQO.EditAuthItemValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //新增权限项
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuName(params.getMenuName());
        menuDTO.setMenuId(params.getMenuId());
        menuDTO.setMenuType(MenuTypeEnum.ROLE.getCode());
        setNormalFieldValue(menuDTO, record);
        menuService.updatePage(menuDTO);

        String operationItemIds = params.getOperationItemIds();
        String[] operationItemArr = operationItemIds.split(SignConstants.COMMA);
        //校验操作项是否存在
        List<Integer> menuIdList = new ArrayList<>(operationItemArr.length);
        for (String str : operationItemArr) {
            menuIdList.add(Integer.parseInt(str));
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        if (menuList == null || menuList.size() < operationItemArr.length) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作项数据有误，其中有不存在的操作项");
        }
        //新增权限项、操作项关联表数据
        List<MenuItemDTO> itemDTOList = new ArrayList<>();
        Date now = new Date();
        for (String operationItem : operationItemArr) {
            MenuItemDTO menuItemDTO = new MenuItemDTO();
            menuItemDTO.setItemId(Integer.parseInt(operationItem));
            menuItemDTO.setMenuId(params.getMenuId());
            menuItemDTO.setStatus(1);
            menuItemDTO.setCreateTime(now);
            menuItemDTO.setUpdateTime(now);
            itemDTOList.add(menuItemDTO);
        }
        //delete
        menuItemService.deleteByMenuId(params.getMenuId());
        //save
        menuItemService.saveFetch(itemDTOList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，没有页面");
        }
        List<AuthItemListDTO> authItemList = menuService.listAuthItemByMenuId(menuId);
        return ServiceResponse.success(authItemList);
    }

    /**
     * 查询gic基础服务版本列表数据
     * @Title: getGicDefaultVersion 
     * @Description:
      * @author guojuxing  
     * @return  java.util.Map<java.lang.String,java.lang.String> 
      */
    @Override
    public ServiceResponse<Map<String, String>> getGicDefaultVersion() {
        Map<String, String> map = new HashMap<>(16);
        ServiceResponse<List<ServePropDTO>> versionListResult = serveApiService.listServePropByGICDefaultAPP();
        if (versionListResult.isSuccess()) {
            List<ServePropDTO> versionList = versionListResult.getResult();
            if (CollectionUtils.isNotEmpty(versionList)) {
                for (ServePropDTO servePropDTO : versionList) {
                    if ("version".equals(servePropDTO.getPropType())) {
                        map.put(servePropDTO.getCode(), servePropDTO.getName());
                    }
                }
            }
        }
        return ServiceResponse.success(map);
    }

    private ServiceResponse<Void> savePage(MenuDTO menuDTO, Class clazz) {
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        setMenuVersion(menuDTO);
        setNormalFieldValue(menuDTO);
        menuService.savePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Integer> saveChildPage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        setNormalFieldValue(menuDTO, record);
        if (!MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            //如果是应用子页面、操作项
            if (StringUtils.isBlank(menuDTO.getMenuVersionName())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用版本名称不能改为空");
            }
        }
        setMenuVersion(menuDTO);

        Integer menuId = menuService.savePage(menuDTO);
        return ServiceResponse.success(menuId);
    }

    private ServiceResponse<Void> updatePage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz) {
        return updateNormalPage(menuDTO, menuTypeEnum, clazz, true);
    }

    private ServiceResponse<Void> updateChildPage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz) {
        return updateNormalPage(menuDTO, menuTypeEnum, clazz, false);
    }

    private ServiceResponse<Void> updateNormalPage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz,
            boolean isFirstLevelPage) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (isFirstLevelPage) {
            if (record.getLevel().intValue() != MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
            }
        } else {
            if (record.getLevel().intValue() == MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非子页面");
            }
        }
        if (record.getMenuType().intValue() != menuTypeEnum.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非" + menuTypeEnum.getMessage());
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }

        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), menuDTO.getMenuId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        menuDTO.setProject(record.getProject());
        if (!MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            //如果是应用子页面、操作项
            if (StringUtils.isBlank(menuDTO.getMenuVersionName())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用版本名称不能改为空");
            }
        }
        setMenuVersion(menuDTO);
        menuService.updatePage(menuDTO);

        //同时更新子集的父级code数据
        menuService.updateParentCodeByParentId(menuDTO.getMenuId(), menuDTO.getMenuCode());

        return ServiceResponse.success();
    }

    /**
     * 应用第一级页面新增
     */
    private void saveAppMenu() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        Map<String, TabSysMenu> appMenu = getFirstLevelAppMenu();
        if (appListResponse.isSuccess()) {
            List<ApplicationDTO> appList = appListResponse.getResult();
            for (ApplicationDTO app : appList) {
                if (appMenu.containsKey(app.getApplicationId().toString())) {
                    //如果已存在，则更新版本信息
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), menuDTO);
                    menuDTO.setMenuId(appMenu.get(app.getApplicationId().toString()).getMenuId());
                    menuService.updatePage(menuDTO);
                } else {
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), menuDTO);
                    setNormalFieldValue(menuDTO);
                    menuService.savePage(menuDTO);
                }
            }
        }
    }

    /**
     * 商户已购买的应用，变成一级菜单
     * @Title: saveHasBuyAppMenu 
     * @Description:
      * @author guojuxing
     * @param hasBuyAppList 
     * @return  void 
      */
    private void saveHasBuyAppMenu(List<ApplicationDTO> hasBuyAppList) {
        Map<String, TabSysMenu> appMenu = getFirstLevelAppMenu();
        for (ApplicationDTO app : hasBuyAppList) {
            if (appMenu.containsKey(app.getApplicationId().toString())) {
                //如果已存在，则更新版本信息
                MenuDTO menuDTO = new MenuDTO();
                menuDTO.setMenuName(app.getName());
                menuDTO.setProject(app.getApplicationId().toString());
                menuDTO.setMenuId(appMenu.get(app.getApplicationId().toString()).getMenuId());
                menuService.updatePage(menuDTO);
            } else {
                MenuDTO menuDTO = new MenuDTO();
                menuDTO.setMenuName(app.getName());
                menuDTO.setProject(app.getApplicationId().toString());
                setNormalFieldValue(menuDTO);
                menuService.savePage(menuDTO);
            }
        }
    }

    /**
     * 第一级页面通用字段值设置
     * @param menuDTO
     */
    private void setNormalFieldValue(MenuDTO menuDTO) {
        //sort
        int maxSort;
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, menuDTO.getProject());
        } else {
            maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, null);
        }
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(1);

        menuDTO.setParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID);
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
    }

    /**
     * 新增子页面、操作项、权限项的时候设置常规字段值
     * @param menuDTO
     * @param parent 父级页面
     */
    private void setNormalFieldValue(MenuDTO menuDTO, TabSysMenu parent) {
        //project
        menuDTO.setProject(parent.getProject());
        //sort
        if (menuDTO.getMenuId() == null) {
            //新增的时候需要设置sort和level。编辑的时候不需要，因为编辑的时候不能再次选择新的父级页面，所以层级是不变的。
            int maxSort = menuService.getMaxSortByParentId(menuDTO.getParentId(), menuDTO.getProject());
            menuDTO.setSort(maxSort + 1);
            //level
            menuDTO.setLevel(parent.getLevel() + 1);
        }
        //父级code
        menuDTO.setParentCode(parent.getMenuCode());
    }

    /**
     * 获取应用第一级菜单（应用列表）
     * @return
     */
    private Map<String, TabSysMenu> getFirstLevelAppMenu() {
        MenuListQO params = new MenuListQO();
        params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        params.setLevel(MenuLevelConstants.FIRST_LEVEL);
        List<TabSysMenu> menuList = menuService.listMenu(params);
        Map<String, TabSysMenu> map = new HashMap<>(16);
        if (menuList != null) {
            for (TabSysMenu menu : menuList) {
                map.put(menu.getProject(), menu);
            }
        }
        return map;
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param serviceList
     * @param menuDTO
     */
    private static void setMenuVersion(List<ServePropDTO> serviceList, MenuDTO menuDTO) {
        if (serviceList == null) {
            return;
        }
        StringBuilder menuVersion = new StringBuilder(SignConstants.UNDERLINE);
        StringBuilder menuVersionName = new StringBuilder();
        for (ServePropDTO propDTO : serviceList) {
            menuVersion.append(propDTO.getCode()).append(SignConstants.UNDERLINE);
            menuVersionName.append(propDTO.getName()).append(SignConstants.SLASH);
        }
        menuDTO.setMenuVersion(menuVersion.toString());
        String tempMenuVersionName = menuVersionName.toString();
        menuDTO.setMenuVersionName(tempMenuVersionName.substring(0, tempMenuVersionName.length() - 1));
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param menuDTO 必须有project值
     */
    private void setMenuVersion(MenuDTO menuDTO) {
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            String[] menuVersionArr = menuDTO.getMenuVersion().split(SignConstants.UNDERLINE);
            StringBuilder result = new StringBuilder();


            Map<String, String> gicDefaultVersion = getGicDefaultVersion().getResult();
            for (String str : menuVersionArr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                result.append(gicDefaultVersion.get(str)).append(SignConstants.SLASH);
            }
            String menuVersionStr = result.toString();
            menuDTO.setMenuVersionName(menuVersionStr.substring(0, menuVersionStr.length() - 1));
        } else {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            menuDTO.setMenuVersionName(
                    menuDTO.getMenuVersionName().replaceAll(SignConstants.COMMA, SignConstants.SLASH));
        }
    }

    private List<MenuDTO> treeMenu(List<TabSysMenu> menuList, int level, int parentId) {
        List<MenuDTO> list = null;
        if (level < 5) {
            for (TabSysMenu menu : menuList) {
                if (level == menu.getLevel() && menu.getParentId().intValue() == parentId) {
                    MenuDTO menuDTO = EntityUtil.changeEntityByJSON(MenuDTO.class, menu);
                    menuDTO.setChildren(this.treeMenu(menuList, menu.getLevel().intValue() + 1, menuDTO.getMenuId()));
                    if (list == null) {
                        list = new ArrayList<>();
                    }
                    list.add(menuDTO);
                }
            }
        }
        return list;
    }

    /**
     * 获取所有的默认操作项（即所有用户都有权限的操作项）, 默认获取gic
     * @Title: getDefaultOperationItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.dto.MenuDTO> 
      */
    private List<MenuDTO> getDefaultOperationItem(Integer isGIC) {
        //返回结果集
        List<MenuDTO> resultList = new ArrayList<>();
        //查询权限项
        MenuListQO params = new MenuListQO();
        params.setMenuType(MenuTypeEnum.ROLE.getCode());
        if (isGIC == null) {
            params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        }
        List<TabSysMenu> authItemList = this.menuService.listMenu(params);
        //查询所有已配置到权限项的操作项
        List<Integer> authItemIdList = new ArrayList<>();
        for (TabSysMenu menu : authItemList) {
            authItemIdList.add(menu.getMenuId());
        }
        List<TabSysMenuItem> menuItemList = menuItemService.listMenuItemByMenuId(authItemIdList);

        Map<Integer, String> menuItemMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(menuItemList)) {
            for (TabSysMenuItem menuItem : menuItemList) {
                menuItemMap.put(menuItem.getItemId(), "1");
            }
        }

        //查询所有的操作项
        params.setMenuType(MenuTypeEnum.OPERATION.getCode());
        List<TabSysMenu> operationItemList = this.menuService.listMenu(params);
        //与已配置的操作项比对，如果没有配置在权限项，说明是默认的操作项（全部用户拥有该操作项）
        for (TabSysMenu sysMenu : operationItemList) {
            if (!menuItemMap.containsKey(sysMenu.getMenuId())) {
                //如果不存在，说明是默认的
                resultList.add(EntityUtil.changeEntityNew(MenuDTO.class, sysMenu));
            }
        }
        return resultList;
    }

    /**
     * 获取配置在权限项内的操作项数据
     * @Title: getOperationItemOfAuthItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.dto.MenuDTO> 
      */
    private List<MenuDTO> getOperationItemOfAuthItem() {
        List<Integer> operationItemIdList = menuItemService.getOperationItemIdList();
        List<TabSysMenu> operationItemList = menuService.selectByIds(operationItemIdList);
        return EntityUtil.changeEntityListNew(MenuDTO.class, operationItemList);
    }

    /**
     * 查询用户拥有的权限菜单(页面、权限项)
     * @Title: getUserMenuList 
     * @Description: 
      * @author guojuxing 
     * @param userId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    private List<TabSysMenu> getUserMenuList(Integer userId) {
        //查询用户的权限集
        List<UserRoleDTO> userRoleList = userRoleService.listUserRoleByUserId(userId);
        List<Integer> roleIdList = new ArrayList<>(userRoleList.size());
        for (UserRoleDTO userRole : userRoleList) {
            roleIdList.add(userRole.getRoleId());
        }
        //查询用户拥有的菜单
        List<MenuRoleDTO> menuRoleList = menuRoleService.listMenuRoleByRoleId(roleIdList);
        List<Integer> menuIdList = new ArrayList<>(menuRoleList.size());
        for (MenuRoleDTO menuRole : menuRoleList) {
            menuIdList.add(menuRole.getMenuId());
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        return menuList;
    }

    /**
     * 获取登陆用户页面菜单
     * @Title: getUserLoginMenu 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param currentMenu
     * @param isGic  是否是gic项目
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    private ServiceResponse<List<MenuDTO>> getUserLoginMenu(Integer userId, Integer currentMenu, boolean isGic) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }

        List<TabSysMenu> resultList;

        if (tabSysUser.getSuperAdmin().intValue() == 1) {
            //超级用户
            MenuListQO params = new MenuListQO();
            params.setMenuType(MenuTypeEnum.PAGE.getCode());
            if (isGic) {
                params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
            } else {
                params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
            }
            resultList = this.menuService.listMenu(params);
        } else {
            //用户拥有的菜单权限
            List<TabSysMenu> menuList = getUserMenuList(userId);
            List<Integer> menuIdList = new ArrayList<>();
            for (TabSysMenu menu : menuList) {
                boolean isMapMenu;
                if (isGic) {
                    isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()
                            && menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
                } else {
                    isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()
                            && !menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
                }
                if (isMapMenu) {
                    menuIdList.add(menu.getMenuId());
                }
            }
            resultList = menuService.selectByIds(menuIdList);
        }
        int level = 1;
        int parentId = 0;
        if (currentMenu != null) {
            TabSysMenu menu = this.menuService.getMenuById(currentMenu);
            parentId = currentMenu;
            level = menu.getLevel() + 1;
        }
        return ServiceResponse.success(this.treeMenu(resultList, level, parentId));
    }

    /**
     * 获取用户没有权限的操作项
     * @Title: getNoAuthOperationItem 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param projectCode 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    private ServiceResponse<List<MenuDTO>> getNoAuthOperationItem(Integer userId, int projectCode) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }
        //用户拥有的菜单权限
        List<TabSysMenu> menuList = getUserMenuList(userId);

        //获取用户拥有的权限项ID
        List<Integer> authItemIdList = new ArrayList<>();
        for (TabSysMenu menu : menuList) {
            boolean isMapMenu;
            if (projectCode == MenuProjectConstants.DEFAULT_PROJECT_CODE) {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()
                        && menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
            } else if (projectCode == MenuProjectConstants.APP_PROJECT_CODE) {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()
                        && !menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
            } else {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode();
            }
            if (isMapMenu) {
                authItemIdList.add(menu.getMenuId());
            }
        }

        List<MenuDTO> resultList = new ArrayList<>();
        Map<Integer, Object> operationItemMap = new HashMap<>(16);
        //查询用户拥有的操作项
        List<TabSysMenuItem> menuItemList = menuItemService.listMenuItemByMenuId(authItemIdList);
        if (CollectionUtils.isNotEmpty(menuItemList)) {
            for (TabSysMenuItem menuItem : menuItemList) {
                operationItemMap.put(menuItem.getItemId(), "1");
            }
        }

        List<MenuDTO> operationItemListOfAuthItem = getOperationItemOfAuthItem();
        for (MenuDTO menuDTO : operationItemListOfAuthItem) {
            if (!operationItemMap.containsKey(menuDTO.getMenuId())) {
                //如果不存在，说明是没有权限的操作项
                resultList.add(menuDTO);
            }
        }
        //仅超管可用
        if (tabSysUser.getSuperAdmin().intValue() != 1) {
            List<TabSysUser> adminOnlyOperationItemList = menuService.listAdminOnlyOperationItem();
            if (CollectionUtils.isNotEmpty(adminOnlyOperationItemList)) {
                resultList.addAll(EntityUtil.changeEntityListNew(MenuDTO.class, adminOnlyOperationItemList));
            }
        }
        return ServiceResponse.success(resultList);
    }
}
