package com.gic.auth.web.controller;

import java.util.ArrayList;
import java.util.List;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AccountGroupMemberTypeEnum;
import com.gic.auth.constant.SignConstants;
import com.gic.auth.dto.AccountGroupDTO;
import com.gic.auth.dto.AccountGroupMemberDTO;
import com.gic.auth.dto.CollaboratorListDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.qo.CollaboratorListQO;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.AccountGroupApiService;
import com.gic.auth.service.CollaboratorApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.vo.AccountGroupListVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/account-group")
@Validated
public class AccountGroupController {
    private final static Logger LOGGER = LogManager.getLogger(AccountGroupApiService.class);
    @Autowired
    private AccountGroupApiService accountGroupApiService;
    @Autowired
    private CollaboratorApiService collaboratorApiService;
    @Autowired
    private UserApiService userApiService;

    /**
     * 新增账号分组
     * @Title: saveAccountGroup 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/save")
    public RestResponse saveAccountGroup(AccountGroupDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(accountGroupApiService.save(dto));
    }

    @RequestMapping("/edit")
    public RestResponse editAccountGroup(AccountGroupDTO dto) {
        return ResultControllerUtils.commonResult(accountGroupApiService.update(dto));
    }

    @RequestMapping("/list")
    public RestResponse listAccountGroup() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(accountGroupApiService.listAccountGroupByEnterpriseId(enterpriseId),
                AccountGroupListVO.class);
    }

    @RequestMapping("/list-no-member")
    public RestResponse listAccountGroupOfNoMember() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(accountGroupApiService.listNoMemberByEnterpriseId(enterpriseId),
                AccountGroupListVO.class);
    }

    @RequestMapping("/delete")
    public RestResponse deleteAccountGroup(Integer accountGroupId) {
        return ResultControllerUtils.commonResult(accountGroupApiService.deleteByAccountGroupId(accountGroupId));
    }

    @RequestMapping("/sort")
    public RestResponse sortAccountGroup(Integer accountGroupId, Integer sort) {
        return ResultControllerUtils.commonResult(accountGroupApiService.sort(accountGroupId, sort));
    }

    /**
     * 全部账号数量
     * @Title: allGroupMember 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/all-group-member")
    public RestResponse allGroupMember() {
        int groupMemberCount = 0;
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        UserListQO userListQO = new UserListQO();
        userListQO.setEnterpriseId(enterpriseId);
        ServiceResponse<Page<UserListDTO>> userResult = userApiService.pageUser(userListQO);
        if (userResult.isSuccess()) {
            groupMemberCount += userResult.getResult().getTotalCount();
        }
        CollaboratorListQO collaboratorListQO = new CollaboratorListQO();
        collaboratorListQO.setEnterpriseId(enterpriseId);
        ServiceResponse<Page<CollaboratorListDTO>> collaboratorResult = collaboratorApiService.page(collaboratorListQO);
        if (collaboratorResult.isSuccess()) {
            groupMemberCount += collaboratorResult.getResult().getTotalCount();
        }
        return RestResponse.success(groupMemberCount);
    }

    /**
     * 不在分组下的组员
     * @Title: memberNotInGroup 
     * @Description: 
      * @author guojuxing 
     * @param accountGroupId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/member-not-in-group")
    public RestResponse memberNotInGroup(Integer accountGroupId, String search) {
        return ResultControllerUtils
                .commonResult(accountGroupApiService.listNotInAccountGroupByAccountGroupId(accountGroupId, search));
    }

    @RequestMapping("/member-in-group")
    public RestResponse memberInGroup(Integer accountGroupId, String search) {
        return ResultControllerUtils
                .commonResult(accountGroupApiService.listInAccountGroupByAccountGroupId(accountGroupId, search));
    }

    @RequestMapping("/grouping")
    public RestResponse grouping(@NotBlank(message = "账号分组ID不能为空") String accountGroupIds,
            @NotBlank(message = "组员ID不能为空") String userIds, @NotNull(message = "组员类型不能为空") Integer type) {
        String[] accountGroupIdArr = accountGroupIds.split(SignConstants.COMMA);
        String[] userIdArr = userIds.split(SignConstants.COMMA);
        List<Integer> accountGroupIdList = new ArrayList<>(accountGroupIdArr.length);
        for (String str : accountGroupIdArr) {
            accountGroupIdList.add(Integer.parseInt(str));
        }
        List<Integer> userIdList = new ArrayList<>(userIdArr.length);
        for (String str : userIdArr) {
            userIdList.add(Integer.parseInt(str));
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (type.intValue() == AccountGroupMemberTypeEnum.ADMIN.getCode()) {
            return ResultControllerUtils
                    .commonResult(accountGroupApiService.groupingOfUser(userIdList, accountGroupIdList, enterpriseId));
        } else if (type.intValue() == AccountGroupMemberTypeEnum.COLLABORATOR.getCode()) {
            return ResultControllerUtils.commonResult(
                    accountGroupApiService.groupingOfCollaborator(userIdList, accountGroupIdList, enterpriseId));
        }
        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "组员类型错误");
    }

    /**
     * 批量新增组员 ，必须有 必须有分组ID、管理员/协作人ID、类型
     * @Title: addGroupMember 
     * @Description:
      * @author guojuxing
     * @param memberArr 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/add-group-member")
    public RestResponse addGroupMember(String memberArr) {
        List<AccountGroupMemberDTO> list = JSON.parseArray(memberArr, AccountGroupMemberDTO.class);
        if (CollectionUtils.isNotEmpty(list)) {
            Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
            for (AccountGroupMemberDTO temp : list) {
                temp.setEnterpriseId(enterpriseId);
            }
        }
        return ResultControllerUtils
                .commonResult(accountGroupApiService.saveAccountGroupRelFetch(list));
    }
}
