package com.gic.auth.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditedGroupDTO;
import com.gic.auth.qo.AuditedGroupQO;
import com.gic.auth.service.AuditedGroupApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-18 18:04
 */
@RestController
public class AuditedGroupController {

    @Autowired
    private AuditedGroupApiService auditedGroupApiService;

    @RequestMapping("/save-audited-group")
    public RestResponse saveOrUpdateAuditedGroup(@Validated(AuditedGroupQO.SaveValid.class) AuditedGroupQO auditedGroupQO) {
        AuditedGroupDTO auditedGroupDTO = EntityUtil.changeEntityByJSON(AuditedGroupDTO.class, auditedGroupQO);
        auditedGroupDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        List<String> list = Arrays.asList(auditedGroupQO.getUserIds().split(GlobalInfo.FLAG_COMMA));
        List<Integer> ids = list.stream().map(Integer::valueOf).collect(Collectors.toList());
        auditedGroupDTO.setUserIdList(ids);
        ServiceResponse<Integer> response = auditedGroupApiService.saveOrUpdateAuditedGroup(auditedGroupDTO);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/del-audited-group")
    public RestResponse delAuditedGroup(@Validated(AuditedGroupQO.DelValid.class) AuditedGroupQO auditedGroupQO) {
        ServiceResponse<Integer> response = auditedGroupApiService.delAuditedGroup(auditedGroupQO.getAuditedGroupId());
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/get-audited-group")
    public RestResponse getAuditGroup(@Validated(AuditedGroupQO.DelValid.class) AuditedGroupQO auditedGroupQO) {
        ServiceResponse<AuditedGroupDTO> response = auditedGroupApiService.getAuditedGroup(auditedGroupQO.getAuditedGroupId());
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/list-audited-group")
    public RestResponse listAuditedGroup(AuditedGroupQO auditedGroupQO) {
        auditedGroupQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<AuditedGroupDTO>> response = auditedGroupApiService.listAuditedGroup(auditedGroupQO);
        return ResultControllerUtils.commonResult(response);
    }
    @RequestMapping("/list-all-audited-group")
    public RestResponse listAllAuditedGroup() {
        ServiceResponse<List<AuditedGroupDTO>> response = auditedGroupApiService.listAllAuditedGroup(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response);
    }
}
