package com.gic.auth.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.qo.AuditorListQO;
import com.gic.auth.service.AuditorApiService;
import com.gic.auth.web.qo.AuditorQO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.ProjectDTO;
import com.gic.enterprise.service.ProjectApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-20 10:31
 */
@RestController
public class AuditorController {

    private Logger logger = LogManager.getLogger(AuditorController.class);

    @Autowired
    private AuditorApiService auditorApiService;
    @Autowired
    private ProjectApiService projectApiService;

    @RequestMapping("/save-auditor")
    public RestResponse saveAuditor(@Validated(AuditorQO.SaveValid.class) AuditorQO auditorQO) {
        AuditorDTO auditorDTO = EntityUtil.changeEntityByJSON(AuditorDTO.class, auditorQO);
        auditorDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        List<Integer> projectItemList = Stream.of(auditorQO.getProjectItems().split(GlobalInfo.FLAG_COMMA)).map(Integer::valueOf).collect(Collectors.toList());
        auditorDTO.setProjectItemList(projectItemList);
        List<Integer> auditedGroupIdList = Stream.of(auditorQO.getAuditedGroupIds().split(GlobalInfo.FLAG_COMMA)).map(Integer::valueOf).collect(Collectors.toList());
        auditorDTO.setAuditedGroupIdList(auditedGroupIdList);
        ServiceResponse<String> response = auditorApiService.saveOrUpdateAuditor(auditorDTO);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/del-auditor")
    public RestResponse delAuditor(Integer auditorId) {
        ServiceResponse<Void> response = auditorApiService.delAuditor(auditorId);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/get-auditor")
    public RestResponse getAuditor(Integer auditorId) {
        ServiceResponse<AuditorDTO> response = auditorApiService.getAuditor(auditorId);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/list-auditor")
    public RestResponse pageAuditor(AuditorListQO auditorListQO) {
        auditorListQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<AuditorDTO>> response = auditorApiService.pageAuditor(auditorListQO);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/list-auditor-nopage")
    public RestResponse listAuditor(AuditorListQO auditorListQO) {
        auditorListQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<List<AuditorDTO>> response = auditorApiService.listAuditor(auditorListQO);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/list-all-project")
    public RestResponse listProject() {
        ServiceResponse<List<ProjectDTO>> serviceResponse = projectApiService.listAllProject();
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
