package com.gic.auth.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.OrderResourceChannelEnum;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.web.qo.OrderResourceQO;
import com.gic.auth.web.qo.PageQO;
import com.gic.auth.web.qo.ResourceQO;
import com.gic.auth.web.qo.StoreResouceQO;
import com.gic.auth.web.vo.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.dto.GoodsRightsSelectorDTO;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.service.sms.SmsApiService;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.dto.ApplicationSubDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.ApplicationSubApiService;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/29 9:09
 */
@RestController
public class ResourceController {

    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;
    @Autowired
    private SmsApiService smsApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private ApplicationSubApiService applicationSubApiService;

    @RequestMapping("/list-resource-app")
    public RestResponse listResourceApp() {
        Integer enterpriseId = 100;
        ServiceResponse<List<ApplicationDTO>> appServiceResponse = appTokenApiService.listApplicationByEnterpriseId(Long.valueOf(enterpriseId), 0);
        if (!appServiceResponse.isSuccess()) {
            return ResultControllerUtils.commonResult(appServiceResponse);
        }
        List<ApplicationDTO> appDTOList = appServiceResponse.getResult();
        List<ApplicationVO> appVOList = new ArrayList<>(appDTOList.size());
        for (ApplicationDTO applicationDTO : appDTOList) {
            ServiceResponse<List<ApplicationSubDTO>> subServiceResponse = applicationSubApiService.listSubApp(enterpriseId, applicationDTO.getApplicationId());
            if (subServiceResponse.isSuccess()) {
                if (CollectionUtils.isNotEmpty(subServiceResponse.getResult())) {
                    ApplicationVO applicationVO = new ApplicationVO();
                    applicationVO.setApplicationId(applicationDTO.getApplicationId());
                    applicationVO.setName(applicationDTO.getName());
                    applicationVO.setChildren(EntityUtil.changeEntityListByJSON(ApplicationSubVO.class, subServiceResponse.getResult()));
                    appVOList.add(applicationVO);
                }
            }
        }
        return RestResponse.success(appVOList);
    }

    @RequestMapping("/list-sms-sign")
    public RestResponse listSmsSign(String search) {
        ServiceResponse<List<EnterpriseSmsSignDTO>> serviceResponse = smsApiService.listSignByEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId(), search);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/save-resource")
    public RestResponse saveResource(@RequestBody ResourceQO resourceQO) {
        ResourceDTO resourceDTO = EntityUtil.changeEntityByJSON(ResourceDTO.class, resourceQO);
        resourceDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        // 门店资源
        ServiceResponse<Integer> storeWidgetResponse = saveStoreWidget(resourceQO.getStoreResouceJSON());
        if(storeWidgetResponse.isSuccess()) {
            resourceDTO.setStoreResource(storeWidgetResponse.getResult().longValue());
        }
        GoodsRightsSelectorDTO goodsRightsSelectorDTO = EntityUtil.changeEntityByJSON(GoodsRightsSelectorDTO.class, resourceQO.getGoodsResource());
        ServiceResponse<GoodsRightsSelectorDTO> goodsResponse = this.goodsRightsSelectorApiService.saveGoodsRightsSelector(goodsRightsSelectorDTO);
        if(goodsResponse.isSuccess()){
            resourceDTO.setGoodsResourceId(goodsResponse.getResult().getGoodsRightsSelectorId());
        }
            // 订单资源
        if (CollectionUtils.isNotEmpty(resourceQO.getOrderResourceJSON())) {
            List<OrderResourceQO> orderResource = resourceQO.getOrderResourceJSON();
            JSONArray orderResourceArray = new JSONArray();
            for (OrderResourceQO orderResourceQO : orderResource) {
                List<Integer> storeIds;
                if (OrderResourceChannelEnum.OFFLINE.getCode().equals(orderResourceQO.getChannel())) {
                    ServiceResponse<Integer> widgetResponse = saveStoreWidget(orderResourceQO.getStoreResouceJSON());
                    Integer storeId = widgetResponse.getResult();
                    storeIds = Collections.singletonList(storeId);
                } else {
                    storeIds = orderResourceQO.getStoreContent();
                }
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("channel", orderResourceQO.getChannel());
                jsonObject.put("storeContent", storeIds);
                orderResourceArray.add(jsonObject);
            }
            resourceDTO.setOrderResourceJson(JSON.toJSONString(orderResourceArray));
        }
        // 应用资源
        if (CollectionUtils.isNotEmpty(resourceQO.getAppResourceJSON())) {
            resourceDTO.setAppResourceJson(JSON.toJSONString(resourceQO.getAppResourceJSON()));
        }
        // 短信签名资源
        if (CollectionUtils.isNotEmpty(resourceQO.getSmsSignResourceJSON())) {
            resourceDTO.setSmsSignResourceJson(JSON.toJSONString(resourceQO.getSmsSignResourceJSON()));
        }

        ServiceResponse<Integer> response = resourceApiService.saveOrUpdateResource(resourceDTO);
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("/del-resource")
    public RestResponse delResource(Integer resourceId) {
        ServiceResponse<Integer> serviceResponse = resourceApiService.delResource(resourceId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-resource")
    public RestResponse listResource(ResourceQO resourceQO, PageQO pageQO) {
        ResourceDTO resourceDTO = EntityUtil.changeEntityByOrika(ResourceDTO.class, resourceQO);
        resourceDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<ResourceDTO>> serviceResponse = resourceApiService.listResource(resourceDTO, pageQO.getCurrentPage(), pageQO.getPageSize());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-resource")
    public RestResponse getResource(Integer resourceId){
        ServiceResponse<ResourceDTO> response = resourceApiService.getResource(resourceId);
        if (response.isSuccess()) {
            ResourceDTO dto = response.getResult();
            ResourceVO vo = EntityUtil.changeEntity(ResourceVO.class, dto);
            StoreResouceVO storeResouceVO = getStoreResouce(dto.getStoreResource().intValue());
            vo.setStoreResouce(storeResouceVO);
            if (StringUtils.isNotBlank(dto.getOrderResourceJson())) {
                List<OrderResourceVO> orderResourceList = JSON.parseArray(dto.getOrderResourceJson(), OrderResourceVO.class);
                for (OrderResourceVO orderResourceVO : orderResourceList) {
                    Integer channel = orderResourceVO.getChannel();
                    if (OrderResourceChannelEnum.OFFLINE.getCode().equals(channel)) {
                        List<Integer> storeContent = orderResourceVO.getStoreContent();
                        StoreResouceVO orderStoreResouce = getStoreResouce(storeContent.get(0));
                        if (orderStoreResouce != null) {
                            orderResourceVO.setStoreResouce(orderStoreResouce);
                        }
                    }
                }
                vo.setOrderResourceJSON(orderResourceList);
            }
            if (StringUtils.isNotBlank(dto.getAppResourceJson())) {
                List<List> appResourceList = JSON.parseArray(dto.getAppResourceJson(), List.class);
                vo.setAppResourceJSON(appResourceList);
            }
            ServiceResponse<GoodsRightsSelectorDTO> goodsResponse = this.goodsRightsSelectorApiService.getGoodsRightsSelector(response.getResult().getGoodsResourceId());
            if(goodsResponse.isSuccess()){
                if(goodsResponse.getResult() != null){
                    vo.setGoodsResouce(EntityUtil.changeEntityByJSON(GoodsResouceVO.class, goodsResponse.getResult()));
                }
            }
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(response);
    }

    private StoreResouceVO getStoreResouce(Integer storeWidgetId) {
        StoreResouceVO storeResouceVO = null;
        ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(storeWidgetId);
        if(storeWidget.isSuccess()) {
            if (storeWidget.getResult() != null) {
                storeResouceVO = new StoreResouceVO();
                storeResouceVO.setStoreResourceId(storeWidgetId);
                storeResouceVO.setAuthMode(storeWidget.getResult().getAuthMode());
                storeResouceVO.setSearchParam(storeWidget.getResult().getSearchParam());
                storeResouceVO.setWriteBackParam(storeWidget.getResult().getWriteBackParam());
            }
        }
        return storeResouceVO;
    }

    private ServiceResponse<Integer> saveStoreWidget(StoreResouceQO storeResouce) {
        StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();
        storeWidgetDTO.setAuthMode(storeResouce.getAuthMode());
        storeWidgetDTO.setSearchParam(storeResouce.getSearchParam());
        storeWidgetDTO.setStoreWidgetId(storeResouce.getStoreResourceId());
        storeWidgetDTO.setWriteBackParam(storeResouce.getWriteBackParam());
        return this.storeWidgetApiService.saveAuthStoreWidget(storeWidgetDTO);
    }

}
